import matplotlib.pyplot as plt
import numpy as np
from mpl_toolkits.basemap import Basemap
from mpl_toolkits.axes_grid1 import make_axes_locatable
import rpy2.robjects as ro
from rpy2.robjects.packages import importr
import matplotlib.cm as cm
from scipy import stats


def plot_rectangle(bmap, lonmin,lonmax,latmin,latmax, col):
    """Plot a regtangle on a map"""
    xs = [lonmin,lonmax,lonmax,lonmin,lonmin]
    ys = [latmin,latmin,latmax,latmax,latmin]
    bmap.plot(xs, ys,latlon = True, color=col)

def plotfield(ax, lon, lat, field, title, cmin, cmax, cmap):
    """Plot maps in the equatorial Atlantic"""
    m = Basemap(projection='cyl', llcrnrlat=-14.5,urcrnrlat=14.5,\
            llcrnrlon=-60,urcrnrlon=20,
                resolution='l',ax=ax)
    ax.set_title(title, fontsize=8)
    # draw coastlines
    m.drawcoastlines(linewidth=0.5)
    #drw countries
    m.drawcountries(linewidth=0.25)
    # draw the edge of the map projection region (the projection limb)
    lon2d, lat2d = np.meshgrid(lon, lat)
    x, y = m(lon2d,lat2d)
    ncol=20
    ntick=4
    contour=np.array(range(ncol+1))*(float(cmax-cmin)/ncol)+cmin
    coltick=np.array(range(ntick+1))*(float(cmax-cmin)/ntick)+cmin
    cs=m.contourf(x,y,field, contour, cmap=cmap)
    
    plot_rectangle(m, 0, 15, -14, -3, col="Grey")
    plot_rectangle(m, -40, 0, 10, 14, col="Blue")
    plot_rectangle(m, -20, 0, -3, 3, col="Black")
  
    div = make_axes_locatable(ax)
    cax = div.append_axes("bottom", size="15%", pad=0.05)

    #cbar = m.colorbar(cs,location='bottom',pad="10%") #,fraction=0.3) #,shrink=2)
    cbar = plt.colorbar(cs, orientation="horizontal",cax=cax,ticks=coltick) #cs,location='bottom',pad="10%") #,fraction=0.3) #,shrink=2)
    #cbar.ax.set_aspect(0.5)
    #cbar.set_ticks(coltick)
    #cbar.ax.set_aspect(300)
    cbar.ax.tick_params(labelsize=8)
    #cbar.set_label('hPa')
    

def plotscatter(ax, var1, var2, iobslst, title, xmin, xmax, ymin, ymax):
    """scatter plot of two variables and linear regression"""
    #define R function for linar fit
    ro.r('''
    linear_fit <- function(x, y) {
    library(s2dverification)
    x=array(x)
    y=array(y)
    #
    lm.out <- lm(y ~ x, na.action = na.omit)
    #print(summary(lm.out))
    lm.out
    }
    ''')
    r_linear_fit = ro.r['linear_fit']

    cols = cm.Paired(np.linspace(0, 1, len(list(var1))))
    ax.set_title(title)
    ax.axis([xmin, xmax, ymin,ymax])
    ntick=4
    xtick=np.array(range(ntick+1))*(float(xmax-xmin)/ntick)+xmin
    ax.set_xticks(xtick)

    r_var1=ro.FloatVector(var1)
    r_var2=ro.FloatVector(var2)
    fit=r_linear_fit(r_var1,r_var2)[0]
    cor=stats.pearsonr(r_var1,r_var2)
    x=np.array(range(11))*(float(xmax-xmin)/10)+xmin
    ax.plot(x,x*fit[1]+fit[0], color="black", linewidth = 1)
    print(cor[1])
    if cor[1]<0.005:
        signi="*"
    else:
        signi=""
    ax.annotate("{0:.2f}".format(cor[0])+signi, (xmin+(xmax-xmin)*0.75,ymin+(ymax-ymin)*0.8))
    ax.scatter(var1,var2 , color=cols) 
    #plot vertical and horizontal line for observed values
    ax.plot([xmin,xmax],[var2[iobslst]]*2, linewidth = 2, color="Grey")
    ax.plot([var1[iobslst]]*2, [ymin,ymax], linewidth = 2, color="Grey")
    #plot the observation in black and Grey
    ax.scatter(var1[iobslst[0]],var2[iobslst[0]], color="Black", s=25) #, markersize=2)
    ax.scatter(var1[iobslst[1]],var2[iobslst[1]], color="Grey", s=25) #, markersize=2)
    
def EOFChangeSign(eofmap):
    """check if the sign of the EOF in the tropical Atlantic
    eof: 3D field containing the EOF on the first dimension and the latitude, 
    longitude on second and third"""
    #check if the sign of the first eof is positive in Atl3 for EOF1 
    field=eofmap
    if(np.mean(field[0,12:17,41:59])<0):
        field[0,:,:]=-eofmap[0,:,:]
        #check if the sign of the seconf eof is positive in beng for EOF2 and EOF3
    if(np.nanmean(field[1,1:11,61:74])>0):
        field[1,:,:]=-eofmap[1,:,:]
    if(np.nanmean(field[2,1:11,61:74])>0):
        field[2,:,:]=-eofmap[2,:,:]
    return(field)
    
    
def mmm(fieldlst,iobslst,checkeofsign=False):
    """calculate the multi model mean excuding the observations"""
    mmm=0
    nmod=0
    for iexp,field in enumerate(fieldlst):
        if iexp not in iobslst:
            #check if the sign of the EOF
            if(checkeofsign):
                #check if the sign of the first eof is positive in Atl3 for EOF1 
                if(np.mean(field[0,12:17,41:59])<0):
                    field[0,:,:]=-field[0,:,:]
                #check if the sign of the seconf eof is positive in beng for EOF2 and EOF3
                if(np.nanmean(field[1,1:11,61:74])>0):
                    field[1,:,:]=-field[1,:,:]
                if(np.nanmean(field[2,1:11,61:74])>0):
                    field[2,:,:]=-field[2,:,:]
                    print(iexp)
            mmm=mmm+field
            nmod=nmod+1
    mmm=mmm/nmod
    return(mmm)
