import sys
import os
from glob import glob
import time
import datetime
import cdo as cdop
import netCDF4 as nc
from dateutil.relativedelta import *
import numpy
import shutil
from common import *
import rpy2.robjects as ro
from rpy2.robjects.packages import importr

class Data:
    """This classe define the set of data for the observation or model
    on which will be calculate the statistics.
    The variables are the following:
    - datanames: names of experiments or observational set (string)
    - varname: name of the variables on which the statitics must be calculated (string)
    - inputdir: directory where are stored the original data (string)
    - outpudir: directory to write the output data (string)
    - filelist: list of file on which the calculations must be done (list of string)
    - sdates: list of startdate associated to each file (list of date)
    - filetype: type of the files in Filelist (string)
    - freq: frequency between 2 timesteps (timedelta)
    - nmenber: number of member (int)"""
    def __init__(self, dataname, varname):
        self._dataname=dataname
        self._varname=varname
        self._inputdir=""
        self._outputdir=""
        self._filelist=[]
        self._sdates=[]
        self._filetype=""
        self._freq=-1
        self._nmember=-1
        
    
    ######## set ########

    def set_dataname(self, dataname):
        self._dataname=dataname

    def set_varname(self, varname):
        self._varname=varname
   
    def set_inputdir(self, inputdir):
        self._inputdir=inputdir

    def set_outputdir(self, outputdir):
        self._outputdir=outputdir

    def set_filelist(self, filelist):
        self._filelist=[]
        self._filelist.extend(filelist)

    def set_sdates(self, sdates):
        self._sdates=[]
        self._sdates.extend(sdates)

    def set_filetype(self, filetype):
        self._filetype=filetype

    def set_freq(self,freq):
        self._freq=freq

    def set_nmember(self, nmember):
        self._nmember = nmember

    ######## get ########

    def get_dataname(self):
        return self._dataname

    def get_varname(self):
        return self._varname
   
    def get_inputdir(self):
        return self._inputdir

    def get_outputdir(self):
        return self._outputdir

    def get_filelist(self):
        return self._filelist

    def get_sdates(self):
        return self._sdates

    def get_filetype(self):
        return self._filetype

    def get_freq(self):
        return self._freq

    def get_nmember(self):
        return self._nmember          

    def ready(self, date1, date2):
        """ Make the Data object ready to calculate statistics """
        #self.set_inputdirAuto() #look for the output directory
        self.set_outputdirAuto() #look for the output directory
        #self.set_files(date1, date2)
        self.format_files_nc() #set file calendar and frequency
        #self
       

    ######## print ########

    def see(self):
        """ print the data object"""
        print "dataname: ", self.get_dataname()
        print "varname: ", self.get_varname()
        print "inputdir: ", self.get_inputdir()
        print "outputdir: ", self.get_outputdir()
        print "filelist: ", self.get_filelist()
        print "sdates: ", self.get_sdates()
        print "filetype: ", self.get_filetype()
        print "freq: ", self.get_freq()
        print "nmember: ", self.get_nmember()


    def duplicate(self):
        """duplicate all atribute execpt the lists"""
        exp2=Data("","")
        exp2.set_varname(self.get_varname())
        exp2.set_dataname(self.get_dataname())
        exp2.set_freq(self.get_freq())
        exp2.set_inputdir(self.get_inputdir())
        exp2.set_outputdir(self.get_outputdir())
        exp2.set_nmember(self.get_nmember())
        return exp2

    def copy(self, aux):
        """copy all atributes of an object in self"""
        self.set_varname(aux.get_varname())
        self.set_dataname(aux.get_dataname())
        self.set_freq(aux.get_freq())
        self.set_inputdir(aux.get_inputdir())
        self.set_outputdir(aux.get_outputdir())
        self.set_nmember(aux.get_nmember())
        self.set_filelist(list(aux.get_filelist()))
        self.set_sdates(list(aux.get_sdates()))


    ######## set_outputdirAuto ########

    def set_outputdirAuto(self):
        """set the output directory based from 
        the input directory"""
        inputdir=self.get_inputdir()
        print inputdir
        if "/" in  inputdir:
            #remove the last part of the input directory and replace it by statistics
            outputdir="/".join(inputdir.split("/")[:-3])+"/monthly_statistics/"
        else:
            print "The path specify must be an abolute path"
            print "this path is not correct or is empty:"
            print inputdir
            sys.exit(0)
        self.set_outputdir(outputdir)
        if (not os.path.isdir(outputdir)):
            os.mkdir(outputdir)
            print "Created directory:"
            print outputdir


   ######## select_leadtime ########

    def select_leadtime(self, leadtime1, leadtime2):
        """select the time period between leadtime1 and leadtime2
        in all files of the Data object:
        -leadtime1: in month (relativedelta)
        -leadtime2: in month (relativedelta)
        the file must have the correct format for cdo
        (run the format_files_nc method in case of failure)
        (to be changed when the right named convention has been set)
        """
        cdo=cdop.Cdo()
        cdo.tempfile.tempdir = os.environ['TMPDIR']

        #get the needed variables from the object
        filelist=self.get_filelist()
        sdates=self.get_sdates()
        dataname=self.get_dataname()
        varname=self.get_varname()
        freq=self.get_freq()
        #timedelta can not be compared: find a solution
        #if ((leadtime1!=relativedelta()) and (leadtime2 < leadtime1)):
        #        print leadtime1
        #        print leadtime2
        #        print "the second leadtime must be larger than the first one"
        #        sys.exit(0)

        filelistout=[]
        sdatesout=[]
        
        lead1str=str(leadtime1.months).zfill(2)
        lead2str=str(leadtime2.months).zfill(2)
        varout=varname + "_" + lead1str + "-" + lead2str
        for i in range(len(filelist)):
            f=filelist[i]
            d=sdates[i]
            
            fdate1=d+leadtime1
            fdate2=d+leadtime2
            

            print "the selected time period is:"
            print fdate1, fdate2

            #calculate the last date of the file
            lastdate=get_last_date(f)

            #check if the time period is smaller the the length of the file
            
            if (fdate2 > lastdate):
                print fdate2
                print lastdate
                print "the selected time period is not include in the file:"
                print f
                sys.exit(0)


            fileout=(f.split("/")[-1])
            fileout=fileout.split(".")[0]+"_"+lead1str+"-"+lead2str+".nc"
            if len(glob(fileout))==0:
                cdo.chname(varname,varout, input=cdo.seldate(fdate1,fdate2, input=f), output=fileout, options="-r -f nc")
            filelistout.append(fileout)
            sdatesout.append(fdate1)
            
        
        
        exp_season=self.duplicate()
        exp_season.set_varname(varout)
        exp_season.set_filelist(list(filelistout[:]))
        exp_season.set_sdates(sdatesout[:])
        return exp_season


    def format_files_nc(self):
        """this fonction check if the file in the filelist 
        have a proper format.
        get the variable freq
        the calendar is checked and set if needed
        the variables incompatibles with cdo are removed"""
        cdo=cdop.Cdo()
        cdo.tempfile.tempdir = os.environ['TMPDIR']

        #self.see()

        #get the needed variables from the object
        filelist=self.get_filelist()
        sdates=self.get_sdates()
        dataname=self.get_dataname()
        varname=self.get_varname()
        
        
        #caculate the frequency and check in all files 
        for i in range(len(filelist)):
            f=filelist[i]
            d=sdates[i]
            flocal=f.split("/")[-1]
            #copy the file in the scratch (faster)
            if len(glob(flocal))==0:
                shutil.copy(f,".")
                f=flocal
                #calculate  freq
                freq = get_freq(f)

                #test if the taxis axis is well defined
                # add more test if needed
                if freq == 0:
                    freq=self.get_freq()
                    f=settaxis(dataname, varname, f, d, freq)
                
                if get_last_date(f) <= d:
                    f=settaxis(dataname, varname, f, d, freq)
                self.set_freq(freq)
            else:
                f=flocal
            filelist[i]=f

            self.set_freq(get_freq(f))
        self.set_filetype("nc")
        #self.see()

    def create_monthly_list(self):
        """create a list of Data object
        containing only one month"""
        #get the needed variables from the object
        filelist=list(self.get_filelist())
        sdates=list(self.get_sdates())
        dataname=self.get_dataname()
        freq=self.get_freq()
        
        f=filelist[0]
        lastdate=get_last_date(f)
        nbmon=diff_month(lastdate,sdates[0])
        listmondata=[]
        for imon in range(4): #nbmon+1):
            leadtime1=relativedelta(months=+imon)
            leadtime2=relativedelta(months=+imon+1, days=-1)
            listmondata.append(self.select_leadtime(leadtime1, leadtime2))
        return listmondata




    ###################################
    ########    STATISTICS     ########
    ###################################     


    ######## monthly_percentile ########
 
    def monthly_percentile(self, percentile):
        """calculate the monthly percentile for all file in filelist"""

        #cdo=Cdo()

        cdo=cdop.Cdo()
        cdo.tempfile.tempdir = os.environ['TMPDIR']

        #get the needed variables from the object
        filelist=list(self.get_filelist())
        sdates=list(self.get_sdates())
        dataname=self.get_dataname()
        varname=self.get_varname()
        freq=self.get_freq()
        varout=varname + "_q" + str(percentile)
        for i in range(len(filelist)):
            f=filelist[i]
            fileout=(f.split("/")[-1])
            fileout=fileout.split(".")[0]+"_q"+str(percentile)+".nc"
            cdo.chvar(varname,varout, 
                        input="-monpctl,"+str(percentile)+" "+f+" -monmin "+f+" -monmax "+f, 
                        output=fileout,
                        options="-r -f nc")

            filelist[i] = fileout
            #when ncdf4 instaled change name of variables in files


        exp_stat=self.duplicate()
        exp_stat.set_filelist(filelist)
        exp_stat.set_sdates(sdates)
        exp_stat.set_freq = relativedelta(months=+1)
        exp_stat.set_varname(varname + "_q" + str(percentile))
        
        return exp_stat

     ######## monthly_nbday ########

    def monthly_nbday(self, percentile, comp):
        """calculate the monthly percentage of days 
        greater the climatological percentile 
        for all file in filelist.
        Note: the climatological percentile 
        is calculated with all files of filelist
        """
        listmondata=self.create_monthly_list()
        listdatastat=[]
        for data in listmondata:
            datastat=data.seasonal_nbday(percentile, comp)
            listdatastat.append(datastat)

        var0=(listdatastat[0]).get_varname()
        var0=quit_leadtime(var0)
        var0=var0.split("-")
        var1=(listdatastat[-1]).get_varname()
        var1=quit_leadtime(var1)
        var1=var1.split("-")

        varnew=var0[0]+"-"+var1[1]
        expstat=concat_monthly_data(listdatastat,
                                    varnew)
        return expstat

     ######## seasonal_nbday ########


    def seasonal_nbday(self, percentile, comp):
        """calculate the monthly percentile for all file in filelist"""
        #cdo=Cdo()
        
        cdo=cdop.Cdo()
        cdo.tempfile.tempdir = os.environ['TMPDIR']

        #get the needed variables from the object
        filelist=list(self.get_filelist())
        sdates=list(self.get_sdates())
        dataname=self.get_dataname()
        varname=self.get_varname()
        freq=self.get_freq()
        nmember=self.get_nmember()

        #get the name of the variable in the files
        ncdfmer=nc.Dataset(filelist[0], "r")
        variables=ncdfmer.variables.keys()
        allDimNames = ncdfmer.dimensions.keys() 
        #get last date of the last file
        dateend=get_last_date(filelist[-1])
        period=str(sdates[0]).replace("-","")+"-"+str(dateend).replace("-","")
        if varname in variables:
            varmer = ncdfmer.variables[varname]
            varaux=varname
        else:
            varaux=varname.split("_")[0]
            if varaux in variables:
                varmer = ncdfmer.variables[varaux]
            else:
                print "the variables %s or %s is not found in the file:" %varname,varaux
                print filelist[0]
                sys.exit(0)
        
        
        #check if there is no time overlap between the file
        for i in range(len(sdates)-1):
            lastdate = get_last_date(filelist[i])
            if (lastdate > sdates[i+1]):
                sdates[i]=sdates
                print "the file are overlaping"
                sys.exit(0)

        varnew="climq"+str(percentile)+"_"+period+"_"+varname
        #calculate the climatological percentile
        percentfile=self.cdostat_clim(
            "ymonpctl", varnew, 
            auxparam=","+str(percentile))
        #print(percentfile)
        ncdfper=nc.Dataset(percentfile, "r")
        percent=ncdfper.variables[varnew]
        shapper=shape(percent)
        #percent=numpy.resize(percent,(shapper[2],shapper[3]))
        percent=numpy.resize(percent,(shapper[1],shapper[2]))
        #ncdfper.close()
        filelistout=[]
        for f in filelist:
            print "treating file:", f
            ncdfin=nc.Dataset(f, "r")
            varin=ncdfin.variables[varaux]
            shapin=shape(varin)
            leadtime=shapin[0]
            if not(len(shapin) == 4):
                if not(len(shapin) == 4):
                    if (len(shapin) == 3):
                        varin=numpy.resize(varin,(shapin[0],1,shapin[1],shapin[2]))
                        shapin=shape(varin)
                    else:
                        print "the file does not have 3 or 4 dimensions"
                        sys.exit(0)
            if not(shapin[1] ==  nmember):
                print "the number of member in the file is not correct"
                sys.exit(0)
            
            varmercomp=numpy.array([percent,]*nmember)
            varout=numpy.zeros((shapin[1],shapin[2],shapin[3]), dtype='f')

            for t in range(leadtime):
                if comp == "greater":
                    varout=varout+numpy.greater_equal(varin[t,:,:,:],varmercomp)
                if comp == "less":
                    varout=varout+numpy.less_equal(varin[t,:,:,:],varmercomp)

            varout=varout/leadtime
            if numpy.amax(varout) > 1:
                print "the percent of day is greater than 1"
                print shape(varout)
                print leadtime
                sys.exit(0)
            #convert the output in a standart python type
            fileout=f.split("/")[-1]
            fileout=fileout.split(".")[0]+"_nbdays.nc"
            ncdfout=nc.Dataset(fileout, "w")
            
            write_lonlat(ncdfout,ncdfin)
            
            ncdfout.createDimension('time', None)
            if "reftime" in ncdfin.variables.keys():
                timevar="reftime"
            elif "time" in ncdfin.variables.keys():
                timevar="time"
            else:
                print "no time variable found"
                sys.exit(0)
            #create the reftime variable
            reftime=ncdfout.createVariable(timevar, "i", ('time',))
            reftime[:] = [0]
            reftimein = ncdfin.variables[timevar]
            reftime.units = reftimein.units
            reftime.standard_name = "forecast_reference_time"
            reftime.longname = "forecast reference time"
            reftime.calendar=reftimein.calendar

            ncdfout.createDimension('ensemble', nmember)
            
            var=ncdfout.createVariable(varname, 'f', ("time", "ensemble", "latitude", "longitude",))
            var[:]=varout
            ncdfout.close()
            ncdfin.close()
            #ncdfper.close()
            filelistout.append(fileout)
        ncdfper.close()
        exp_stat=self.duplicate()
        exp_stat.set_filelist(filelistout)
        exp_stat.set_sdates(sdates)
        exp_stat.set_freq = relativedelta(months=+1)

        #to be changed when final convention defined!!!!!
        exp_stat.set_varname(varname + "_nbdays_q" + str(percentile) + "_" + period)
        return exp_stat

     ######## seasonal_percentile ########


    def seasonal_percentile(self, percentile):
        """calculate the monthly percentile over the whole period
        for all file in filelist
        arguments:
        -percentile: percentile to be calculated 
        outputs:
        - object of type data with the computed index
        """

        #cdo=Cdo()
        
        cdo=cdop.Cdo()
        cdo.tempfile.tempdir = os.environ['TMPDIR']

        #get the needed variables from the object
        filelist=list(self.get_filelist())
        sdates=list(self.get_sdates())
        dataname=self.get_dataname()
        varname=self.get_varname()
        freq=self.get_freq()
        
        for i in range(len(filelist)):
            f=filelist[i]
            fileout=(f.split("/")[-1])
            fileout=fileout.split(".")[0]+"_q"+str(percentile)+".nc"
            cdo.timpctl(percentile, input=f+" -timmin "+ f+" -timmax "+f, 
                        output=fileout,
                        options="-r -f nc")

            filelist[i] = fileout

        
        exp_stat=self.duplicate()
        exp_stat.set_filelist(filelist)
        exp_stat.set_sdates(sdates)
        exp_stat.set_freq = relativedelta(months=+1)
        #to be changed when final convention defined!!!!!
        exp_stat.set_varname(varname + "_q" + str(percentile) )
       
        return exp_stat



     ######## daily statistics ########
    def cdo_statistic(self, stat, varout):
        """calcultate a cdo statistic: stat
        on all file of filelist
        -stat: statistics to be applied
        this statistic must have one only input
        which is the input file
        -name of the output varible
        """
        #cdo=Cdo()

        cdo=cdop.Cdo()
        cdo.tempfile.tempdir = os.environ['TMPDIR']

        #get the needed variables from the object
        filelist=self.get_filelist()
        sdates=list(self.get_sdates())
        dataname=self.get_dataname()
        varname=self.get_varname()
        freq=self.get_freq()
       
        for i in range(len(filelist)):
            f=filelist[i]
            if f!="":
                fileout=(f.split("/")[-1])
                fileout=fileout.split(".")[0]+varout+".nc"
                if len(glob(fileout))==0:
                    cdo.chvar(varname,varout, 
                              input=" -"+stat+" "+f, output=fileout)

                filelist[i] = fileout
            else:
                f=""
        #print filelist

        #create a copy of the  
        #self.set_filelist(filelist)
        self.set_freq(get_freq(filelist[0]))
        self.set_varname(varout)

    def calc_reg(self, indexname, fldmean=True, mask=True):
        """calculate an index for all files
        of the data object
        arguments:
        - name of the index 
        (has to be known by the mask_reg function)
        outputs:
        - object of type data with the computed index
        """

        #get the needed variables from the object
        filelist=self.get_filelist()
        sdates=list(self.get_sdates())
        dataname=self.get_dataname()
        varname=self.get_varname()
        freq=self.get_freq()
        
        exp_stat=self.duplicate()
        fileoutlst=[]
        for f in filelist:
            fout=compute_reg(f, dataname, indexname, varname, fldmean=fldmean, mask=mask)
            fileoutlst.append(fout)
        
        exp_stat.set_sdates(sdates)
        exp_stat.set_filelist(fileoutlst)
        exp_stat.set_varname(varname+"_"+indexname)
       
        return(exp_stat)

    def calc_quantile(self, value, option, crossvalid=False):
        """estimate for file object which contain
        time serie (output of calc_reg with the fldmean option set to True) the quantile
        correponding to a given value. 
        if option quantile:
        arguments:
        - value: the value
        output:
        - the quantile associated to the value.
        if option value:
        arguments:
        - the quantile.
        output:
        - the value associated to a quantile.
        """
    
        #cdo=Cdo()

        cdo=cdop.Cdo()
        cdo.tempfile.tempdir = os.environ['TMPDIR']

        #get the needed variables from the object
        filelist=list(self.get_filelist())
        sdates=list(self.get_sdates())
        dataname=self.get_dataname()
        varname=self.get_varname()
        freq=self.get_freq()
        nmember=self.get_nmember()
        

        year1=sdates[0].year
        year2=sdates[-1].year
        

        if crossvalid!=False:
            try:
                crossindex=sdates.index(crossvalid)
            except:
                print "The startdate " + str(crossvalid)  + " is not in the sdates list of the following object"
                self.see()
                sys.exit(1)
            period=str(year1)+"-"+str(year2)+"_cv"+str(crossvalid)
            filelist.pop(crossindex)
            sdates.pop(crossindex)

    #check if there is no time overlap between the file
        for i in range(len(sdates)-1):
            lastdate = get_last_date(filelist[i])
            if (lastdate > sdates[i+1]):
                sdates[i]=sdates
                print "the file are overlaping"
                #sys.exit(0)
        merge=cdo.mergetime(input=filelist, options="-O -r -f nc")
        
        #open merge file
        ncdfmer=nc.Dataset(merge, "r")
        variables=ncdfmer.variables.keys()
        allDimNames = ncdfmer.dimensions.keys() 

        varmer = ncdfmer.variables[varname]
        shapmer=varmer.shape
        varmer=numpy.resize(varmer,(shapmer[0]*shapmer[1]))
        shapmer=varmer.shape


        if option == "quantile":
            aux=numpy.where(varmer>value)[0]
            #print(aux)
            #print(numpy.shape(aux))
            quantile=float((numpy.shape(aux))[0])/((numpy.shape(varmer))[0])
            #print(numpy.shape(varmer))
            return(quantile)
        if option == "value":
            aux=numpy.percentile(varmer,value)
            return(aux)


   
    def calc_onset_thres(self, thres):
        """return the number of the first timestep of 
        the file of filelist exeeding a thres old
        can be use for the estimation of the monsoon onset
        -thres: the threshold
        return the list of output files
        """
    
        #cdo=Cdo()

        cdo=cdop.Cdo()
        cdo.tempfile.tempdir = os.environ['TMPDIR']

        #get the needed variables from the object
        filelist=list(self.get_filelist())
        sdates=list(self.get_sdates())
        dataname=self.get_dataname()
        varname=self.get_varname()
        freq=self.get_freq()
        nmember=self.get_nmember()     
        outputdir=self.get_outputdir()

        fileoutlst=[]
        if type(thres) is float:
            thres=[thres]*len(filelist)
        for ifile in range(len(filelist)):
            f=filelist[ifile]
            d=sdates[ifile]

            #calculate the onset
            onset=calc_fst_thres(f,thres[ifile], varname)
            #convert to float to avoid trouble when writing
            onset=numpy.array(onset,  dtype='float32')

            varout="onset_"+varname+"_thres_cv"
            dirout=os.path.join(outputdir,"seasonal_statistics",varout)
            if (not os.path.isdir(dirout)):
                print(dirout)
                os.makedirs(dirout)
 
            fileout=varout+"_"+str(d).replace("-","")+".nc"
            fileout=os.path.join(dirout,fileout)
            #print(fileout)
            #print(nmember)
            ncdfout=nc.Dataset(fileout, "w")
            ncdfout.createDimension('ensemble', nmember)
            var=ncdfout.createVariable(varout, 'f', ("ensemble",))
            
            var.longname="onset date calculated over 1993 and 2009 for the two first month of the hindcast based on value in the model corresponding to the quantile computed in cross validation associated to 5mm/day in ERAint over this period"
            var.standard_name="onset"
            var.units="nb of day from 1st of May"
            #print(onset)
            fileoutlst.append(fileout)
            if(onset.shape==(1,)):
                var[:]=onset[0]
            else:
                var[:]=onset
            ncdfout.close()
        return(fileoutlst)


    def daily_anom(self, sd=False):
        """calculate the daily anomalies"""
        #cdo=Cdo()
        
        cdo=cdop.Cdo()
        cdo.tempfile.tempdir = os.environ['TMPDIR']

        #get the needed variables from the object
        filelist=list(self.get_filelist())
        sdates=list(self.get_sdates())
        dataname=self.get_dataname()
        varname=self.get_varname()
        freq=self.get_freq()
        nmember=self.get_nmember()

        
        
        #check if there is no time overlap between the file
        for i in range(len(sdates)-1):
            lastdate = get_last_date(filelist[i])
            if (lastdate > sdates[i+1]):
                sdates[i]=sdates
                print "the file are overlaping"
                sys.exit(0)
        clim=cdo.ydaymean(input="-del29feb -vertmean "+cdo.mergetime(
                input=filelist, options="-O -r -f nc"),
                          output="clim.nc", options="-O -r -f nc -b 32")
        
        if(sd):
            flstaux=[]
            for imemb in range(nmember):
                for f in filelist:
                    flstaux.append(cdo.sellevidx(imemb+1, input=f))
            sd=" "+cdo.ydaystd(input="-del29feb "+cdo.mergetime(
                    input=flstaux, options="-O -r -f nc"),
                              output="sd.nc", options="-O -r -f nc -b 32")

            #define string to cadenate cdo operations
            strsd="-div "
            sdname="sd"
        else:
            sdname=""
            sd=""
            strsd=""
        print "the merge of file has been done successfully"

        filelistout=[]
        year1=sdates[0].year
        year2=sdates[-1].year
        
        period=str(year1)+"-"+str(year2)
        varnew=varname+"_"+sdname+"anom_"+period
        for f in filelist:
            print "treating file:", f
            
            faux=cdo.chvar(varname, varnew, 
                           input=strsd+"-sub -del29feb -daymean "+f+" "+clim+sd, 
                           output="anom_"+period+"_"+f,
                           options="-O -r -f nc -b 32")
            filelistout.append(faux)
            
        exp_stat=self.duplicate()
        exp_stat.set_filelist(filelistout)
        exp_stat.set_sdates(sdates)
        exp_stat.set_freq = datetime.timedelta(0, 0, 0, 0, 0, 24)

        exp_stat.set_varname(varnew)
        return exp_stat


    def daily_clim(self, ensmean=True):
        """calculate the daily climatology"""
        #cdo=Cdo()
        
        cdo=cdop.Cdo()
        cdo.tempfile.tempdir = os.environ['TMPDIR']

        #get the needed variables from the object
        filelist=list(self.get_filelist())
        sdates=list(self.get_sdates())
        dataname=self.get_dataname()
        varname=self.get_varname()
        freq=self.get_freq()
        nmember=self.get_nmember()
        outputdir=self.get_outputdir()
        outputdir=self.get_outputdir().replace("monthly_statistics", "daily_clim")
        
        period=str(sdates[0]).replace("-","")+"-"+str(sdates[-1]).replace("-","")
        
        
        #check if there is no time overlap between the file
        for i in range(len(sdates)-1):
            lastdate = get_last_date(filelist[i])
            if (lastdate > sdates[i+1]):
                sdates[i]=sdates
                print "the file are overlaping"
                sys.exit(0)
        varnew=varname+"_dayclim_"+period
        fileout=outputdir+varnew+".nc"
        if(ensmean):
             ensmean=" -vertmean"
        else:
             ensmean=" "
                 
        cdo.chvar(varname,varnew, input="-ydaymean -del29feb"+ensmean+cdo.mergetime(
                input=filelist, options="-O -r -f nc"),
                          output=fileout, options="-O -r -f nc -b 32")
        
        return fileout
        
        
    def cdostat_clim(self, cdostat, varnew, auxparam="", crossvalid=False):
        """calculate the daily climatology
        - cdostat: name of the statististics
        - varnew: name of the new variable
        - auxparam: additional parameter need for the stat
        - crossvalid: if not false, exclude the given starting date from the climatology
        """
        #cdo=Cdo()
        
        cdo=cdop.Cdo()
        cdo.tempfile.tempdir = os.environ['TMPDIR']

        #get the needed variables from the object
        filelist=list(self.get_filelist())
        sdates=list(self.get_sdates())
        dataname=self.get_dataname()
        varname=self.get_varname()
        freq=self.get_freq()
        nmember=self.get_nmember()
        outputdir=self.get_outputdir()
        year1=sdates[0].year
        year2=sdates[-1].year
        
        period=str(year1)+"-"+str(year2)
        #remove the given year for cross_validation
        if crossvalid!=False:
            try:
                crossindex=sdates.index(crossvalid)
            except:
                print "The startdate " + str(crossvalid)  + " is not in the sdates list of the following object"
                self.see()
                sys.exit(1)
            period=str(year1)+"-"+str(year2)+"_cv"+str(crossvalid)
            filelist=filelist.pop(crossindex)

            
        
        #check if there is no time overlap between the file
        for i in range(len(sdates)-1):
            lastdate = get_last_date(filelist[i])
            if (lastdate > sdates[i+1]):
                sdates[i]=sdates
                print "the file are overlaping"
                sys.exit(0)
        
        fileout=varnew+".nc"

        outputclim=outputdir.replace("monthly_statistics","monthly_clim")
        if (not os.path.isdir(outputclim)):
            os.mkdir(outputclim)

        if len(glob(outputclim+fileout))==0:
            filemerge="merge_"+period+"_"+varname+".nc"
            f=filelist[0]
            fmergeaux="merge_"+f
            if len(glob(filemerge))==0:
                if len(glob(fmergeaux))==0:
                    flstaux=[]
                    for imemb in range(nmember):
                        flstaux.append(cdo.sellevidx(imemb+1, input=f))
                    cdo.mergetime(
                        input=flstaux, output=fmergeaux, 
                        options="-O -r -f nc")  

                shutil.copy(fmergeaux,filemerge)
                for f in filelist[1:]:
                    fmergeaux="merge_"+f
                    if len(glob(fmergeaux))==0:
                        flstaux=[]
                        for imemb in range(nmember):
                            flstaux.append(cdo.sellevidx(imemb+1, input=f))
                        cdo.mergetime(
                            input=flstaux, output=fmergeaux, 
                            options="-O -r -f nc")  
                    filemergeaux=filemerge+"tmp"
                    cdo.mergetime(
                        input=[filemerge,fmergeaux], output=filemergeaux, 
                        options="-O -r -f nc")   
                    shutil.copy(filemergeaux,filemerge)
                    os.remove(filemergeaux)
    
            #print("-"+cdostat+auxparam+" "+filemerge)
            if cdostat=="ymonpctl":
                param=" -monmin "+filemerge+" -monmax "+filemerge
            else:
                param=""
            #convention to be harmonized 
            #(obj and files should have the same variable names)
            varbase=varname.split("_")[0]
            cdo.chvar(varbase,varnew, 
                      input="-"+cdostat+auxparam+" "+filemerge+param,
                      output=fileout, options="-O -r -f nc")
            shutil.copy(fileout,outputclim+fileout)
        
        return outputclim+fileout
        
    #def cdo_sub(self, exp):
    #    """use cdo to substract one by one all the file of two objects
    #    self-exp
    #    """
    def EOF(self, neof):
        """the eof for all the files of the object, return an R object. 
        INPUT
        #####
        self: Can be applied on DataAtl obsjects or object containing a large period
        not adequate for seasonal hindcast except if the file has been merge
        neof: number of EOF to calculate
        OUTPUT
        ######
        List of object (one per file of filelist) containing the PC and the eof field

        """
        #cdo=Cdo()
        
        cdo=cdop.Cdo()
        cdo.tempfile.tempdir = os.environ['TMPDIR']

        #get the needed variables from the object
        filelist=list(self.get_filelist())
        sdates=list(self.get_sdates())
        dataname=self.get_dataname()
        varname=self.get_varname()
        freq=self.get_freq()
        nmember=self.get_nmember()
        #define the R funcion call Cfu eigentechnics
        ro.r('''
            calc_eof <- function(f, var, neof) {
            source("~/tools/cfutools/eigentechniques/CFU_Rfunc.txt")
            library(ncdf)
         
            fnc=open.ncdf(f)
            field=get.var.ncdf(fnc,var)
            ntime=dim(field)[3]
            fieldperm=aperm(field,c(3,2,1))
            lon=get.var.ncdf(fnc,"lon")
            lat=get.var.ncdf(fnc,"lat")
            nlon=length(lon)
            nlat=length(lat)
            eof=CFU_EOF(lon,lat,fieldperm,neofs=neof)
            eof
            }
        ''')
        r_calc_eof = ro.r['calc_eof']
        eoflst=[]
        for f in filelist:
            eof=r_calc_eof(f, varname, neof)
            eoflst.append(eof)
        return(eoflst)

    def monthly_anom(self):
        """calulate the monthly anomalies
        INPUT
        #####
        self: Can be applied on DataAtl obsjects or object containing a large period
        of MONTHLY data
        not adequate for seasonal hindcast except if the file has been merge
        OUTPUT
        ######
        Data object containing monthly anomalies
        """
        #cdo=Cdo()
        
        cdo=cdop.Cdo()
        cdo.tempfile.tempdir = os.environ['TMPDIR']

        
        #get the needed variables from the object
        filelist=list(self.get_filelist())
        sdates=list(self.get_sdates())
        dataname=self.get_dataname()
        varname=self.get_varname()
        freq=self.get_freq()
        nmember=self.get_nmember()
        #define new variable name
        varnew=varname+"_monanom"
        
        #loop over all the files
        filelistout=[]
        for f in filelist:
            #define the name of file out
            fileout=varnew+"_"+f.split("/")[-1]
            cdo.chvar(varname,varnew, 
                      input="-ymonsub "+f+" -ymonmean "+f,
                      output=fileout, options="-O -r -f nc")
            filelistout.append(fileout)
            
        exp_anom=self.duplicate()
        exp_anom.set_filelist(filelistout)
        exp_anom.set_sdates(sdates)
        exp_anom.set_varname(varnew)


        return(exp_anom)
    
    
    def calc_mld(self):
        """compute the mixed layer depth based on the de boyer montegut temerature criteria
        first level where the temperature is 0.2 colder than the 10m temperature
        """
        varnew="mld"
        filelist=self.get_filelist()
        varname=self.get_varname()
        fileoutlst=[]
        for f in filelist:
            fileoutlst.append(mld_dt02(f,varname))
            
        self.set_filelist(fileoutlst)
        self.set_varname("mld")